package gssim.db.mysql;

import gssim.db.dao.data.UnitAllTimeline;
import gssim.db.find.Finder;
import gssim.db.utils.PreparedStatementAdapter;
import gssim.db.utils.ResultSetAdapter;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.LogFactory;

import com.mysql.jdbc.NotImplemented;

public class UnitAllTimelineDAO extends AbstractDAO implements
		gssim.db.dao.UnitAllTimelineDAO {

	protected UnitAllTimelineDAO(Connection conn) throws SQLException {
		super(conn);
		log = LogFactory.getLog(UnitAllTimelineDAO.class);
		setRelationName("unit_allocation_timeline");
	}

	@Override
	protected PreparedStatementAdapter createDeleteStmt() throws SQLException {
		return null;
	}

	@Override
	protected PreparedStatementAdapter createFindStmt() throws SQLException {
		String sql = "SELECT * FROM unit_allocation_timeline WHERE"
				+ " (fk_unit_id = ? OR ISNULL(?))"
				+ " AND (timestamp between ? AND ?)";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createInsertStmt() throws SQLException {
		String sql = "INSERT INTO unit_allocation_timeline VALUES(?, ?, ?)";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));

		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createSelectStmt() throws SQLException {
		String sql = "SELECT * FROM unit_allocation_timeline WHERE fk_unit_id = ? AND timestamp = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createUpdateStmt() throws SQLException {
		return null;
	}

	@Override
	public boolean delete(UnitAllTimeline obj, boolean commit)
			throws SQLException {
		throw new NotImplemented();
	}

	@Override
	public void fillObjectFromResultSet(UnitAllTimeline obj, ResultSet result)
			throws SQLException {
		throw new NotImplemented();

	}

	@Override
	public int fillStatementWithObject(PreparedStatement stmt,
			UnitAllTimeline obj, int offset) throws SQLException {
		throw new NotImplemented();
	}

	@Override
	public List<UnitAllTimeline> find(Finder finder) throws SQLException {

		PreparedStatementAdapter stmt = getFindStmt();
		List<UnitAllTimeline> list = new ArrayList<UnitAllTimeline>();

		stmt.setInt(1, finder.getInteger("fkUnitId"));
		stmt.setInt(2, finder.getInteger("fkUnitId"));
		stmt.setTimestamp(3, finder.getMinTimestamp("timestamp"));
		stmt.setTimestamp(4, finder.getMaxTimestamp("timestamp"));

		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		while (result.next()) {

			UnitAllTimeline obj = new UnitAllTimeline();

			Integer fkUnitId = result.getIntObj("fk_unit_id");
			Timestamp timestamp = result.getTimestamp("timestamp");
			Float usage = result.getFloatObj("usage");

			obj.setFkUnitId(fkUnitId);
			obj.setTimestamp(timestamp);
			obj.setUsage(usage);

			list.add(obj);
		}

		return list;
	}

	@Override
	public int insert(UnitAllTimeline obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		stmt.setInt(1, obj.getFkUnitId());
		stmt.setTimestamp(2, obj.getTimestamp());
		stmt.setFloat(3, obj.getUsage());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("insert");
		} else
			logError("insert", result, obj);

		return result;
	}

	@Override
	public int[] insertBatch(List<UnitAllTimeline> objList, boolean commit)
			throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		for (int i = 0; i < objList.size(); i++) {
			stmt.setInt(1, objList.get(i).getFkUnitId());
			stmt.setTimestamp(2, objList.get(i).getTimestamp());
			stmt.setFloat(3, objList.get(i).getUsage());
			stmt.addBatch();
		}

		int[] result = stmt.executeBatch();

		stmt.clearBatch();

		int j = 0;

		while (j < result.length)
			if (result[j] == 1)
				j++;

		if (j == result.length) {
			if (commit)
				commit();

			logSuccess("batch insert", j);

		} else
			logError("batch insert", j);

		return result;
	}

	@Override
	public UnitAllTimeline select(UnitAllTimeline obj) throws SQLException {

		PreparedStatementAdapter stmt = getSelectStmt();
		stmt.setInt(1, obj.getFkUnitId());
		stmt.setTimestamp(2, obj.getTimestamp());
		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		result.next();
		boolean first = result.isFirst();
		boolean last = result.isLast();

		if (first && last) {

			obj.setUsage(result.getFloatObj("usage"));
			return obj;
		}

		// no reservation found with specified task_id and unit_id
		if (!first && !last) {
			return null;
		}

		// no idea how this may happen (only if id is not primary key)
		// no idea how this may happen (only if id is not primary key)
		throw new SQLException("Select operation in relation "
				+ getRelationName() + " returned more than one record");
	}

	@Override
	public boolean update(UnitAllTimeline obj, boolean commit)
			throws SQLException {
		throw new NotImplemented();
	}

}