package gssim.db.utils;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.math.BigInteger;
import java.security.InvalidKeyException;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.RSAPrivateKeySpec;
import java.security.spec.RSAPublicKeySpec;
import java.util.Properties;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;

/**
 * This program generates a RSA keys, retrieves its raw bytes, and then
 * reinstantiates a RSA keys from the key bytes. The reinstantiated key is used
 * to initialize a AES cipher for encryption and decryption.
 */

public class DbCredentialsManager {

	public static void savePropertiesToEncryptedFile(Properties properties,
			String propertiesFilePath, String keyFilePath)
			throws InvalidKeyException, IllegalBlockSizeException,
			BadPaddingException, IOException {

		FileOutputStream propertiesFileInputStream = new FileOutputStream(
				propertiesFilePath);

		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try {
			properties.store(baos, "Seagrass credentials");
		} catch (IOException e) {
			// should not occur
		}

		byte[] message = baos.toByteArray();
		byte[] result = encrypt(message, keyFilePath);

		try {
			propertiesFileInputStream.write(result);
		} finally {
			propertiesFileInputStream.close();
		}

	}

	public static Properties loadPropertiesFromEncryptedFile(
			String propertiesFilePath, String keyFilePath)
			throws InvalidKeyException, IllegalBlockSizeException,
			BadPaddingException, IOException {
		Properties properties = new Properties();

		File propertiesFile = new File(propertiesFilePath);
		FileInputStream propertiesFileInputStream = new FileInputStream(
				propertiesFile);

		byte[] message = new byte[(int) propertiesFile.length()];
		propertiesFileInputStream.read(message);

		byte[] result = decrypt(message, keyFilePath);
		properties.load(new ByteArrayInputStream(result));
		return properties;
	}

	public static void printProperties() {
		Properties properties = new Properties();
		properties.put("ssss", "dsadas");
		properties.put("dsadas", "dsadsa");

		System.out.println(properties);
	}

	public static byte[] encrypt(byte[] message, String keyFile)
			throws InvalidKeyException, IllegalBlockSizeException,
			BadPaddingException, IOException {
		PublicKey pubKey = readPublicKeyFromFile(keyFile);
		Cipher cipher = null;
		try {
			cipher = Cipher.getInstance("RSA");
		} catch (NoSuchAlgorithmException e) {
			// should not occur
		} catch (NoSuchPaddingException e) {
			// should not occur
		}
		cipher.init(Cipher.ENCRYPT_MODE, pubKey);
		return cipher.doFinal(message);
	}

	public static byte[] decrypt(byte[] message, String keyFile)
			throws IOException, InvalidKeyException, IllegalBlockSizeException,
			BadPaddingException {
		PrivateKey privKey = readPrivateKeyFromFile(keyFile);
		Cipher cipher = null;
		try {
			cipher = Cipher.getInstance("RSA");
		} catch (NoSuchAlgorithmException e) {
			// should not occur
		} catch (NoSuchPaddingException e) {
			// should not occur
		}
		cipher.init(Cipher.DECRYPT_MODE, privKey);
		return cipher.doFinal(message);
	}

	public static void generateKeyPair(String publicKeyFileName,
			String privateKeyFileName, int keySize)
			throws InvalidKeySpecException, IOException {
		KeyPairGenerator kpg = null;
		try {
			kpg = KeyPairGenerator.getInstance("RSA");
		} catch (NoSuchAlgorithmException e) {
			// there is such an algorithm
		}
		kpg.initialize(keySize);
		KeyPair kp = kpg.genKeyPair();
		KeyFactory fact = null;
		try {
			fact = KeyFactory.getInstance("RSA");
		} catch (NoSuchAlgorithmException e) {
			// there is such an algorithm
		}

		RSAPublicKeySpec pub = fact.getKeySpec(kp.getPublic(),
				RSAPublicKeySpec.class);
		RSAPrivateKeySpec priv = fact.getKeySpec(kp.getPrivate(),
				RSAPrivateKeySpec.class);

		saveToFile(publicKeyFileName, pub.getModulus(), pub.getPublicExponent());
		saveToFile(privateKeyFileName, priv.getModulus(), priv
				.getPrivateExponent());

	}

	private static void saveToFile(String fileName, BigInteger mod,
			BigInteger exp) throws IOException {
		ObjectOutputStream oout = new ObjectOutputStream(
				new BufferedOutputStream(new FileOutputStream(fileName)));
		try {
			oout.writeObject(mod);
			oout.writeObject(exp);
		} catch (Exception e) {
			throw new IOException("Unexpected error", e);
		} finally {
			oout.close();
		}
	}

	private static PublicKey readPublicKeyFromFile(String keyFileName)
			throws IOException {
		InputStream in = new FileInputStream(keyFileName);
		ObjectInputStream oin = new ObjectInputStream(new BufferedInputStream(
				in));
		try {
			BigInteger m = (BigInteger) oin.readObject();
			BigInteger e = (BigInteger) oin.readObject();
			RSAPublicKeySpec keySpec = new RSAPublicKeySpec(m, e);
			KeyFactory fact = KeyFactory.getInstance("RSA");
			PublicKey pubKey = fact.generatePublic(keySpec);
			return pubKey;
		} catch (Exception e) {
			throw new RuntimeException("Spurious serialisation error", e);
		} finally {
			oin.close();
		}
	}

	private static PrivateKey readPrivateKeyFromFile(String keyFileName)
			throws IOException {
		InputStream in = new FileInputStream(keyFileName);
		ObjectInputStream oin = new ObjectInputStream(new BufferedInputStream(
				in));
		try {
			BigInteger m = (BigInteger) oin.readObject();
			BigInteger e = (BigInteger) oin.readObject();
			RSAPrivateKeySpec keySpec = new RSAPrivateKeySpec(m, e);
			KeyFactory fact = KeyFactory.getInstance("RSA");
			PrivateKey privKey = fact.generatePrivate(keySpec);
			return privKey;
		} catch (Exception e) {
			throw new RuntimeException("Spurious serialisation error", e);
		} finally {
			oin.close();
		}
	}

	public static void main(String[] args) throws Exception {

		System.out.println("A. Generate keys");
		System.out.println("B. Encrypt properties file");
		System.out.println("C. Decrypt properties file");
		System.out.print("Please enter your choice: ");

		BufferedReader bufferedReader = new BufferedReader(
				new InputStreamReader(System.in));

		while (true) {
			String choice = bufferedReader.readLine().trim().toUpperCase();
			if ("A".equals(choice)) {
				generationUI();
				break;
			} else if ("B".equals(choice)) {
				encryptionUI();
				break;
			} else if ("C".equals(choice)) {
				decryptionUI();
				break;
			} else
				System.out.print("Incorrect choice, please choose again: ");
		}

	}

	private static void generationUI() throws InvalidKeySpecException,
			IOException {
		BufferedReader bufferedReader = new BufferedReader(
				new InputStreamReader(System.in));
		System.out
				.println("There public and private keys for RSA algorithm will be generated.");
		System.out.print("Please enter public key path: ");
		String publicKeyPath = bufferedReader.readLine();
		System.out.print("Please enter private key path: ");
		String privateKeyPath = bufferedReader.readLine();
		int keySize = 0;
		System.out.println("Please choose key size: 1024 / 2048 / 4096 bits");
		while (true) {
			System.out.print("Key size: ");
			String length = bufferedReader.readLine().trim();
			if ("1024".equals(length)) {
				keySize = 1024;
				break;
			} else if ("2048".equals(length)) {
				keySize = 2048;
				break;
			}
			if ("4096".equals(length)) {
				keySize = 4096;
				break;
			} else
				System.out.println("Invalid key size, please specify again");
		}

		generateKeyPair(publicKeyPath, privateKeyPath, keySize);
		System.out.println("Keys created successfully");
	}

	private static void encryptionUI() throws IOException,
			InvalidKeySpecException, InvalidKeyException,
			IllegalBlockSizeException, BadPaddingException {

		BufferedReader bufferedReader = new BufferedReader(
				new InputStreamReader(System.in));

		System.out
				.print("Please enter unencrypted properties file path (source): ");
		String sourceFilePath = bufferedReader.readLine();
		System.out
				.print("Please enter encrypted properties file path (destination): ");
		String destinationFilePath = bufferedReader.readLine();
		System.out.print("Please enter public key path: ");
		String keyPath = bufferedReader.readLine();

		Properties properties = new Properties();
		properties.load(new FileInputStream(sourceFilePath));
		savePropertiesToEncryptedFile(properties, destinationFilePath, keyPath);

		System.out.println("Encryption performed successfully");
	}

	private static void decryptionUI() throws IOException,
			InvalidKeySpecException, InvalidKeyException,
			IllegalBlockSizeException, BadPaddingException {

		BufferedReader bufferedReader = new BufferedReader(
				new InputStreamReader(System.in));

		System.out
				.print("Please enter encrypted properties file path (source): ");
		String sourceFilePath = bufferedReader.readLine();
		System.out
				.print("Please enter unencrypted properties file path (destination): ");
		String destinationFilePath = bufferedReader.readLine();
		System.out.print("Please enter private key path: ");
		String keyPath = bufferedReader.readLine();

		Properties properties = loadPropertiesFromEncryptedFile(sourceFilePath, keyPath);
		properties.store(new FileOutputStream(destinationFilePath), null);

		System.out.println("Decryption performed successfully");
	}
}
